#version 110

uniform vec2 scales;
uniform sampler3D normals;
uniform samplerCube sky;

varying vec3 position;

// sun parameters
const float sun_strength  = 10.0;
const float sun_shininess = 500.0;
const vec3  sun_color     = vec3(1.0, 0.8, 0.1);
const vec3  sun_direction = normalize(vec3(0.0, -10.0, -1.0));

// refractive color
const vec4 refraction = vec4(0.09, 0.16, 0.3, 1.0);

void main()
{
  // calculate view vector
  vec3 view = normalize(gl_ModelViewMatrixInverse[3].xyz - position);
  
  // fetch normal
  vec3 normal = 2.0 * texture3D(normals, vec3(scales[0] * gl_TexCoord[0].st, 0.0)).xyz - vec3(1.0);
  normal += 2.0 * texture3D(normals, vec3(scales[1] * gl_TexCoord[0].st, 1.0)).xyz - vec3(1.0);
  normal *= 0.5;
  
  // fetch fresnel term
  float f = textureCube(sky, vec3(0.999 * dot(normal, - view), 0.0, -1.0))[0];
  
  // fetch reflection color
  vec3 r = reflect(view, normal);
  vec4 reflection = textureCube(sky, r);
  
  // mix refraction / reflection
  gl_FragColor = mix(refraction, reflection, f);
  
  // add reflected sunlight
  gl_FragColor.rgb += sun_strength * sun_color * pow(max(0.0, dot(r, - sun_direction)), sun_shininess);
}
