#include "Grid.h"

#include <Inventor/nodes/SoCoordinate3.h>
#include <Inventor/nodes/SoIndexedTriangleStripSet.h>

Grid::Grid(SbVec2s size, SbVec2f range)
{
  this->size = size;

  // set up points
  float s = range[1] - range[0], t = range[0];
  SbVec3f * points = new SbVec3f[size[0] * size[1]];
  for (int j = 0; j < size[1]; j++) {
    for (int i = 0; i < size[0]; i++) {
      float x = s * i / (size[0] - 1) + t;
      float y = s * j / (size[1] - 1) + t;
      points[size[0] * j + i] = SbVec3f(x , y, 0.0f);
    }
  }

  // create coordinates
  coords = new SoCoordinate3;
  coords->point.setValues(0, size[0] * size[1], points);
  this->addChild(coords);

  // set up triangle strip indices
  int32_t * indices = new int32_t[(size[1] - 1) * (2 * size[0] + 1)];
  for (int j = 0; j < size[1] - 1; j++) {
    for (int i = 0; i < size[0]; i++) {
      int k = (2 * size[0] + 1) * j + 2 * i;
      indices[k] = size[0] * j + i;
      indices[k + 1] = size[0] * (j + 1) + i;
    }
    indices[(2 * size[0] + 1) * j + 2 * size[0]] = -1; // strip delimiter
  }

  // create triangle strips
  SoIndexedTriangleStripSet * grid = new SoIndexedTriangleStripSet;
  grid->coordIndex.setValues(0, (size[1] - 1) * (2 * size[0] + 1), indices);
  this->addChild(grid);

  // clean up
  delete[] points;
}

SbVec2s
Grid::getSize()
{
  return this->size;
}
