#include "Ocean.h"
#include "WaveField.h"
#include "SkyMap.h"
#include "OceanSurface.h"

#include <Inventor/SbLinear.h>
#include <Inventor/nodes/SoCallback.h>
#include <Inventor/nodes/SoPerspectiveCamera.h>
#include <Inventor/nodes/SoTextureCubeMap.h>
#include <Inventor/nodes/SoTextureUnit.h>

const float speed = 0.02;
const SbVec2s gridsize(129, 129);
const SbVec2f scales(1.0, 0.677);

Ocean::Ocean()
{
  // add camera
  SoPerspectiveCamera * camera = new SoPerspectiveCamera;
  this->addChild(camera);

  // add callback
  SoCallback * cb = new SoCallback;
  cb->setCallback(callback, camera);
  this->addChild(cb);

  // add FFT-based height maps
  WaveField * tiles = new WaveField;
  this->addChild(tiles);

  // change texture unit
  SoTextureUnit * texunit2 = new SoTextureUnit;
  texunit2->unit = 3;
  this->addChild(texunit2);

  // add cube map for environment mapping
  SkyMap * sky = new SkyMap;
  this->addChild(sky);

  // add ocean surface node
  OceanSurface * surface = new OceanSurface(gridsize, scales);
  surface->setCamera(camera);
  this->addChild(surface);

  // adjust camera
  camera->nearDistance = 0.1;
  camera->farDistance = 20.0;
  camera->position = SbVec3f(0.0, 0.0, -0.5);
  camera->orientation.setValue(SbVec3f(1.0, 0.0, 0.0001), 4.6);
}

void
Ocean::callback(void * closure, SoAction *)
{
  SoPerspectiveCamera * camera = (SoPerspectiveCamera *) closure;

  // move the camera
  SbVec3f position = camera->position.getValue();
  position[1] -= speed;
  camera->position.setValue(position);
}
