#include "OceanShader.h"

#include <Inventor/fields/SoMFVec4f.h>
#include <Inventor/nodes/SoVertexShader.h>
#include <Inventor/nodes/SoFragmentShader.h>
#include <Inventor/nodes/SoShaderParameter.h>

OceanShader::OceanShader(SbVec2f scales)
{
  // load shaders
  vert = new SoVertexShader;
  vert->sourceProgram = "VertexShader.glsl";
  vert->ref();

  frag = new SoFragmentShader;
  frag->sourceProgram = "FragmentShader.glsl";
  frag->ref();

  this->shaderObject.set1Value(0, vert);
  this->shaderObject.set1Value(1, frag);

  // initialize shader parameters

  SoShaderParameter1i * texunit0 = new SoShaderParameter1i;
  texunit0->name = "displacements";
  texunit0->value = 0;

  SoShaderParameter1i * texunit1 = new SoShaderParameter1i;
  texunit1->name = "displacements2";
  texunit1->value = 1;

  SoShaderParameter1i * texunit2 = new SoShaderParameter1i;
  texunit2->name = "normals";
  texunit2->value = 2;

  SoShaderParameter1i * texunit3 = new SoShaderParameter1i;
  texunit3->name = "sky";
  texunit3->value = 3;

  frag->parameter.set1Value(0, texunit0);
  frag->parameter.set1Value(1, texunit1);
  frag->parameter.set1Value(2, texunit2);
  frag->parameter.set1Value(3, texunit3);

  SoShaderParameter2f * s = new SoShaderParameter2f;
  s->name = "scales";
  s->value = scales;
  frag->parameter.set1Value(4, s);
}

void
OceanShader::setCorners(SbVec4f c[4])
{
  SoShaderParameter4f * p = new SoShaderParameter4f;
  p->name = "p";
  p->value = c[0];
  vert->parameter.set1Value(0, p);

  SoShaderParameter4f * q = new SoShaderParameter4f;
  q->name = "q";
  q->value = c[1];
  vert->parameter.set1Value(1, q);

  SoShaderParameter4f * r = new SoShaderParameter4f;
  r->name = "r";
  r->value = c[2];
  vert->parameter.set1Value(2, r);

  SoShaderParameter4f * s = new SoShaderParameter4f;
  s->name = "s";
  s->value = c[3];
  vert->parameter.set1Value(3, s);
}

OceanShader::~OceanShader()
{
  // clean up
  vert->unref();
  frag->unref();
}
