#include "OceanSurface.h"
#include "OceanShader.h"

#include "Grid.h"
#include "util.h"

#include <Inventor/nodes/SoCoordinate3.h>
#include <Inventor/nodes/SoPerspectiveCamera.h>
#include <Inventor/sensors/SoNodeSensor.h>

OceanSurface::OceanSurface(SbVec2s resolution, SbVec2f scales)
{
  // add shader program
  shader = new OceanShader(scales);
  this->addChild(shader);

  // add grid
  grid = new Grid(resolution, SbVec2f(0, 1));
  this->addChild(grid);

  // add camera movement sensor
  sensor = new SoNodeSensor(callback, this);
}

void
OceanSurface::setCamera(SoPerspectiveCamera * camera)
{
  // attach sensor to camera
  this->camera = camera;
  sensor->attach(camera);
}

void
OceanSurface::calculateCorners()
{
  // get transformations
  // NOTE: should normally get aspect ratio from viewer
  SbMatrix pm = camera->getViewVolume(1.5).getMatrix();
  SbMatrix pmi = pm.inverse();

  // get frustum
  SbVec4f v[8];
  getFrustum(v, pmi);

  // get intersections between viewing volume and upper/lower planes
  float d = 0.05f; // NOTE: hardcoded d
  SbVec3f intersections[2][12];
  if (!getIntersections(intersections, v, d)) {
    // printf("Nothing to render\n\n");
    // return;
  }

  // get screen-aligned bounding rectangle
  SbBox2f bounds = getBounds(intersections, pm);

  // get corners in world-space, homogeneous coordinates
  SbVec4f c[4];
  getCorners(c, bounds, pmi);

  // update vertex shader with the new corners
  shader->setCorners(c);
}

void
OceanSurface::callback(void * closure, SoSensor * sensor)
{
  ((OceanSurface *) closure)->calculateCorners();
}

OceanSurface::~OceanSurface()
{
  // clean up
  sensor->detach();
  delete sensor;
}
