#include "SkyMap.h"

void sampleFresnel(SoSFImage & image, float ratio);

SkyMap::SkyMap()
{
  // load cube map images
  const char *filenames[] = {
    "skybox_left.png", "skybox_right.png", "skybox_back.png",
    "skybox_front.png", "skybox_bottom.png", "skybox_top.png"
  };
  this->filenames.setValues(0, 6, filenames);

  // hide reflectance function in cube map
  // - for cases of texture unit shortage
  sampleFresnel(this->imageNegZ, 0.75f);
}

void
sampleFresnel(SoSFImage & image, float ratio)
{
  int nc;
  SbVec2s size;
  float ai, at, r;

  unsigned char * bytes = image.startEditing(size, nc);
  int half_w = size[0] / 2, half_h = size[1] / 2;

  for (int i = 0; i < half_w; i++) {
    // get angle of incidence
    ai = acos((float) i / (half_w - 1));

    // get transmission angle by Snell's law
    at = asin(ratio * sin(ai)); 

    // get reflectance by the Fresnel equations
    r = 0.5f * (pow(sin(at - ai) / sin(at + ai), 2) + pow(tan(at - ai) / tan(at + ai), 2));

    // store in byte image
    unsigned char & b = bytes[nc * (size[0] * half_h + half_w + i)];
    b = unsigned char(255.0f * (r > 1.0f ? 1.0f : r));
    bytes[nc * (size[0] * (half_h - 1) + half_w + i)] = b;
  }

  image.finishEditing();
}
