#version 110

uniform vec2 scales;
uniform vec4 p, q, r, s;
uniform sampler2D displacements;
uniform sampler2D displacements2;

varying vec3 position;

const float y_exponent = 0.3;
const vec2 size = vec2(32.0, 32.0);

float linear_lookup(sampler2D sampler, vec2 coord)
{
  vec2 p = floor(size * coord + vec2(0.5));
  vec4 q = vec4(p - vec2(0.5), p + vec2(0.5));
  q.xy /= size;
  q.zw /= size;
  float f[] = {
    texture2D(sampler, vec2(q[0], q[1]))[0],
    texture2D(sampler, vec2(q[2], q[1]))[0],
    texture2D(sampler, vec2(q[0], q[3]))[0],
    texture2D(sampler, vec2(q[2], q[3]))[0]
  };
  p = size * (coord - q.st);
  return dot(vec2(1.0 - p.s, p.s) * mat2(f[0], f[1], f[2], f[3]), vec2(1.0 - p.t, p.t));
}

void main()
{
  float x = gl_Vertex.x, y = pow(gl_Vertex.y, y_exponent);

  // position vertex by bilinear interpolation
  gl_Position = p * (1.0 - x) * (1.0 - y) + q * x * (1.0 - y) + s * (1.0 - x) * y + r * x * y;
  gl_Position /= gl_Position.w;

  // use world space position as vertex coordinate
  gl_TexCoord[0] = gl_Position;

  // displace vertex
  gl_Position.z += linear_lookup(displacements, scales[0] * gl_TexCoord[0].st);
  gl_Position.z += linear_lookup(displacements2, scales[1] * gl_TexCoord[0].st);
  
  // store position
  position = gl_Position.xyz;
  gl_Position = gl_ModelViewProjectionMatrix * gl_Position;
}
