#include "solver.h"

#include <stdlib.h>
#include <GL/glut.h>

#define N 512
#define M 512

Solver *ripples;

void init_glut(char *caption, int width, int height) {
  int x = (glutGet(GLUT_SCREEN_WIDTH) - width) / 2;
  int y = (glutGet(GLUT_SCREEN_HEIGHT) - height) / 2;
  glutInitDisplayMode(GLUT_RGBA | GLUT_DOUBLE);
  glutInitWindowPosition(x, y);
  glutInitWindowSize(width, height);
  glutCreateWindow(caption);
}

void display()
{
  ripples->render();
  glutSwapBuffers();
}

void idle()
{
  ripples->update();
  glutPostRedisplay();
}

void mouse(int button, int state, int x, int y)
{
  y = glutGet(GLUT_WINDOW_HEIGHT) - y - 1;
  x *= float(N) / glutGet(GLUT_WINDOW_WIDTH);
  y *= float(M) / glutGet(GLUT_WINDOW_HEIGHT);

  if (button == GLUT_LEFT_BUTTON && state == GLUT_DOWN) {
    ripples->impulse(x, y, -0.1, 10);
  }
}

void motion(int x, int y)
{
  mouse(GLUT_LEFT_BUTTON, GLUT_DOWN, x, y);
}

int cleanup()
{
  glutDestroyWindow(glutGetWindow());
  delete ripples;
  return 0;
}

int main(int argc, char **argv)
{
  init_glut("Ripples", N, M);

  // register callbacks
  glutDisplayFunc(display);
  glutIdleFunc(idle);
  glutMouseFunc(mouse);
  glutMotionFunc(motion);
  onexit(cleanup);

  glEnable(GL_TEXTURE_2D);
  glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
  glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);

  ripples = new Solver(N, M);

  glutMainLoop();

  return 0;
}
