#include "solver.h"

#include <GL/glut.h>

Solver::Solver(int width, int height)
{
  N = width;
  M = height;

  p = new float[N * M];
  v = new float[N * M];
  a = new float[N * M];
  t = new int[N * M];

  for (int i = 0; i < N * M; i++) {
    p[i] = 0.5;
    v[i] = 0.0;
    a[i] = 0.0;
    t[i] = 0;
  }

  // default values
  friction = 0.999f;
  g_spring = 0.010f;
  n_spring = 0.125f;

  // upload texture
  glTexImage2D(GL_TEXTURE_2D, 0, 1, N, M, 0, GL_LUMINANCE, GL_FLOAT, p);
}

Solver::~Solver()
{
  delete[] p;
  delete[] v;
  delete[] a;
  delete[] t;
}

void Solver::render()
{
  glBegin(GL_QUADS);
  glTexCoord2f(0, 0); glVertex2f(-1, -1);
  glTexCoord2f(1, 0); glVertex2f( 1, -1);
  glTexCoord2f(1, 1); glVertex2f( 1,  1);
  glTexCoord2f(0, 1); glVertex2f(-1,  1);
  glEnd();
}

void Solver::update()
{
  float gv, nv;

  // update buffers
  for (int y = 1; y < M - 1; y++) {
    for (int x = 1; x < N - 1; x++) {
      gv = 0.5 - p[N * y + x];

      nv = -4.0 * p[N * y + x];
      nv += p[N * y + (x - 1)];
      nv += p[N * y + (x + 1)];
      nv += p[N * (y - 1) + x];
      nv += p[N * (y + 1) + x];

      v[N * y + x] += g_spring * gv + n_spring * nv;
    }
  }

  for (int i = 0; i < N * M; i++) {
    p[i] += v[i];
    v[i] *= friction;

    if (t[i]) {
      v[i] += a[i];
      t[i]--;
    }
  }

  // update texture
  glTexSubImage2D(GL_TEXTURE_2D, 0, 0, 0, N, M, GL_LUMINANCE, GL_FLOAT, p);
}

void Solver::impulse(int x, int y, float f, int t)
{
  this->a[N * y + x] = f;
  this->t[N * y + x] = t;
}
