#include "camera.h"

#include <GL/glut.h>
#include <math.h>

Vec3f::Vec3f()
{
  x = y = z = 0.0;
}

Vec3f::Vec3f(float x, float y, float z)
{
  this->x = x;
  this->y = y;
  this->z = z;
}

Vec3f Vec3f::operator+(const Vec3f& p)
{
  return Vec3f(x + p.x, y + p.y, z + p.z);
}

Vec3f Vec3f::operator*(const float& f)
{
  return Vec3f(x * f, y * f, z * f);
}

Camera::Camera()
{
  fovy = 60.0;
  aspect = 4.0 / 3.0;
  zNear = 0.1;
  zFar = 100.0;

  friction = 0.99;
  speed_sensitivity = 0.01;
  spin_sensitivity = 0.001;

  pos = Vec3f(0.0, -2.0, 0.0);

  angle = 0.0;
  spin = 0.0;
  speed = 0.0;
  strafe = 0.0;
  elevate = 0.0;
}

void Camera::view()
{
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  gluPerspective(fovy, aspect, zNear, zFar);

  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();
  Vec3f cen = pos + Vec3f(sin(angle), cos(angle), 0.0);
  gluLookAt(pos.x, pos.y, pos.z, cen.x, cen.y, cen.z, 0.0, 0.0, 1.0);
}

void Camera::update()
{
  float c = cos(angle), s = sin(angle);
  pos = pos + Vec3f(s, c, 0.0) * speed;
  pos = pos + Vec3f(c, -s, 0.0) * strafe;
  pos = pos + Vec3f(0.0, 0.0, elevate);
  
  angle += spin;

  spin *= friction;
  speed *= friction;
  strafe *= friction;
  elevate *= friction;
}

void Camera::input(int key)
{
  if (glutGetModifiers() & GLUT_ACTIVE_ALT) {
    spin = 0.0;
    speed = 0.0;
    strafe = 0.0;
    elevate = 0.0;
    return;
  }

  switch (key) {
    case GLUT_KEY_UP:
      if (glutGetModifiers()) elevate += speed_sensitivity;
      else speed += speed_sensitivity;
      break;
    case GLUT_KEY_DOWN:
      if (glutGetModifiers()) elevate -= speed_sensitivity;
      else speed -= speed_sensitivity;
      break;
    case GLUT_KEY_LEFT:
      if (glutGetModifiers()) strafe -= speed_sensitivity;
      else spin -= spin_sensitivity;
      break;
    case GLUT_KEY_RIGHT:
      if (glutGetModifiers()) strafe += speed_sensitivity;
      else spin += spin_sensitivity;
      break;
  }
}
