uniform vec2 size;
uniform sampler2D sampler;

const float friction = 0.999;
const float g_spring = 0.010;
const float n_spring = 0.125;
const float impulse = -0.015;

const int pos = 0;
const int vel = 1;
const int acc = 2;
const int env = 3;

void main()
{
  vec2 texCoord = gl_TexCoord[0].st;
  const vec2 offset = vec2(1.0) / size;
  
  gl_FragColor = texture2D(sampler, texCoord);
  if (any(lessThanEqual(texCoord, offset)) || any(greaterThanEqual(texCoord, vec2(1.0) - offset)) || bool(gl_FragColor[env])) return;
  
  float gv = 0.5 - gl_FragColor[pos];
  
  float nv = 0.0;
  nv += texture2D(sampler, texCoord + vec2(-offset.x, 0.0))[pos];
  nv += texture2D(sampler, texCoord + vec2(+offset.x, 0.0))[pos];
  nv += texture2D(sampler, texCoord + vec2(0.0, -offset.y))[pos];
  nv += texture2D(sampler, texCoord + vec2(0.0, +offset.y))[pos];
  nv -= 4.0 * gl_FragColor[pos];
  
  // TEMP
  /*
  const float f = 1.0 / sqrt(2.0);
  nv += f * texture2D(sampler, texCoord + vec2(-offset.x, -offset.y))[pos];
  nv += f * texture2D(sampler, texCoord + vec2(-offset.x, +offset.y))[pos];
  nv += f * texture2D(sampler, texCoord + vec2(+offset.x, -offset.y))[pos];
  nv += f * texture2D(sampler, texCoord + vec2(+offset.x, +offset.y))[pos];
  nv -= 4.0 * f * gl_FragColor[pos];
  */
  
  gl_FragColor[vel] += g_spring * gv + n_spring * nv;
  gl_FragColor[vel] += impulse * step(0.01, gl_FragColor[acc]);
  gl_FragColor[pos] += gl_FragColor[vel];
  
  gl_FragColor[vel] *= friction;
  gl_FragColor[acc] = max(0.0, gl_FragColor[acc] - 0.01);
}
