#include "shader.h"

#include <fstream>
using namespace std;

GLuint loadShader(const char *filename)
{
  // identify shader type
  GLenum type;
  const char *ext = strrchr(filename, '.') + 1;
  if (!strcmp(ext, "vert")) type = GL_VERTEX_SHADER;
  else if (!strcmp(ext, "frag")) type = GL_FRAGMENT_SHADER;
  else return 0;

  // open file
  ifstream file(filename, ios::ate | ios::binary);
  if (!file.is_open()) return 0;

  // read entire file
  int size = file.tellg();
  char *src = new char[size];
  file.seekg(0, ios::beg);
  file.read(src, size);

  // set source and compile shader
  GLuint shader = glCreateShader(type);
  glShaderSource(shader, 1, (const char **)&src, &size);
  glCompileShader(shader);

  // clean up and return
  file.close();
  delete[] src;
  return shader;
}

GLuint compileProgram(GLuint vert, GLuint frag)
{
  // create program
  GLuint prog = glCreateProgram();

  // attach shaders
  glAttachShader(prog, vert);
  glAttachShader(prog, frag);

  // link program
  glLinkProgram(prog);

  // check info log
  printProgramInfoLog(prog);

  return prog;
}

void printProgramInfoLog(GLuint prog)
{
  int len;
  glGetProgramiv(prog, GL_INFO_LOG_LENGTH, &len);
  if (len > 0) {
    char *log = new char[len];
    glGetProgramInfoLog(prog, len, &len, log);
    printf("%s\n", log);
    delete[] log;
  }
}
