uniform sampler2D sampler;
uniform sampler1D fresnel;

varying vec3 view;

const float amp = 1.5;

void main()
{
  vec2 texCoord = gl_TexCoord[0].st;
  const vec2 offset = vec2(1.0 / 640.0, 1.0 / 400.0);
  
  vec3 normal;
  normal.x = texture2D(sampler, texCoord + vec2(offset.x, 0.0))[0] -
             texture2D(sampler, texCoord - vec2(offset.x, 0.0))[0];
  normal.y = texture2D(sampler, texCoord + vec2(0.0, offset.y))[0] -
             texture2D(sampler, texCoord - vec2(0.0, offset.y))[0];
  normal.xy *= amp;
  normal.z = - 2.0 * (offset.x + offset.y);
  normal = gl_NormalMatrix * normalize(normal);
  
  // fetch fresnel term (high angle of incidence --> high fresnel)
  float f = texture1D(fresnel, dot(-normal, -view))[0];
  
  vec4 ambient, diffuse, specular;
  gl_LightSourceParameters source = gl_LightSource[0];
  ambient = gl_FrontMaterial.ambient * (source.ambient + gl_LightModel.ambient);
  diffuse = gl_FrontMaterial.diffuse * source.diffuse;
  diffuse *= max(0.0, dot(normal, normalize(source.position.xyz)));
  specular = gl_FrontMaterial.specular * source.specular;
  specular *= pow(max(0.0, dot(normal, source.halfVector.xyz)), gl_FrontMaterial.shininess);
  gl_FragColor = ambient + f * (diffuse + specular) + texture2D(sampler, texCoord)[3];
}
