#include "grid.h"

#include <GL/glew.h>

struct Point {
  float x, y;
};

Grid::Grid(int n, int m, float range_min, float range_max)
{
  this->n = n;
  this->m = m;
  this->strips = new GLuint[m - 1];

  int i, j, k;
  float s = (range_max - range_min) / (n - 1), t = (range_max - range_min) / (m - 1);
  Point *p = new Point[n * m];
  GLuint *e = new GLuint[2 * n];

  // generate data
  glGenBuffers(m - 1, strips);
  for (j = 0; j < m; j++) {
    for (i = 0; i < n; i++) {
      k = n * j + i;
      p[k].x = s * i + range_min;
      p[k].y = t * j + range_min;
      e[2 * i] = k;
      e[2 * i + 1] = k + n;
    }
    if (j < m - 1) {
      // upload strip elements
      glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, strips[j]);
      glBufferData(GL_ELEMENT_ARRAY_BUFFER, 2 * n * sizeof(GLuint), e, GL_STATIC_DRAW);
    }
  }

  // upload point data
  glGenBuffers(1, &points);
  glBindBuffer(GL_ARRAY_BUFFER, points);
  glBufferData(GL_ARRAY_BUFFER, n * m * sizeof(Point), p, GL_STATIC_DRAW);

  delete[] p;
  delete[] e;
}

Grid::~Grid()
{
  glBindBuffer(GL_ARRAY_BUFFER, 0);
  glDeleteBuffers(1, &points);

  glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, 0);
  glDeleteBuffers(m - 1, strips);
  delete[] strips;
}

void Grid::render()
{
  glEnableClientState(GL_TEXTURE_COORD_ARRAY);
  glEnableClientState(GL_VERTEX_ARRAY);
  glEnableClientState(GL_INDEX_ARRAY);

  glBindBuffer(GL_ARRAY_BUFFER, points);
  glTexCoordPointer(2, GL_FLOAT, 0, 0);
  glVertexPointer(2, GL_FLOAT, 0, 0);

  for (int j = 0; j < m - 1; j++) {
    glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, strips[j]);
    glDrawElements(GL_TRIANGLE_STRIP, 2 * n, GL_UNSIGNED_INT, 0);
  }

  glDisableClientState(GL_TEXTURE_COORD_ARRAY);
  glDisableClientState(GL_VERTEX_ARRAY);
  glDisableClientState(GL_INDEX_ARRAY);
}
