uniform sampler2D displacement1;
uniform sampler2D displacement2;
uniform sampler2D normals;
uniform sampler1D fresnel;

varying vec3 view;

const float amp = 2.5;

void main()
{
  const vec2 offset = vec2(1.0 / 512.0, 1.0 / 512.0);
  const float normal_z = - 2.0 * (offset.x + offset.y);
  
  vec2 texCoord = gl_TexCoord[0].st;
  
  vec3 normal;
  normal.x = texture2D(displacement1, texCoord + vec2(offset.x, 0.0))[0] -
             texture2D(displacement1, texCoord - vec2(offset.x, 0.0))[0];
  normal.y = texture2D(displacement1, texCoord + vec2(0.0, offset.y))[0] -
             texture2D(displacement1, texCoord - vec2(0.0, offset.y))[0];
  normal.xy *= amp;
  normal.z = normal_z;
  normal = gl_NormalMatrix * normalize(normal);
  
  vec3 n = 2.0 * texture2D(normals, texCoord).xyz - vec3(1.0);
  normal += gl_NormalMatrix * normalize(vec3(10.0 * n.xy, n.z));
  normal *= 0.5;
  
  // fetch fresnel term (high angle of incidence --> high fresnel)
  float f = texture1D(fresnel, dot(normal, view))[0];
  
  vec4 ambient, diffuse, specular;
  gl_LightSourceParameters source = gl_LightSource[0];
  ambient = gl_FrontMaterial.ambient * (source.ambient + gl_LightModel.ambient);
  diffuse = gl_FrontMaterial.diffuse * source.diffuse;
  diffuse *= max(0.0, dot(normal, normalize(source.position.xyz)));
  specular = gl_FrontMaterial.specular * source.specular;
  specular *= pow(max(0.0, dot(normal, source.halfVector.xyz)), gl_FrontMaterial.shininess);
  gl_FragColor = ambient + f * (diffuse + specular) + texture2D(displacement1, texCoord)[3];
}
