#ifndef TILEMAP_H
#define TILEMAP_H

#include <fftw3.h>

struct vec2 {
  float x, y;

  vec2();
  vec2(float x, float y);
};

float length(const vec2& v);
vec2 normalize(const vec2& v);
float dot(const vec2& u, const vec2& v);

class TileMap {
public:
  TileMap(int N, int M);
  ~TileMap();

  void sample(float amp, float gravity, vec2 wind, float scale = 1.0);
  void sample(float t);
  void ift();

  // upload maps needed by the shaders
  void uploadMaps();

  unsigned int seed;

private:
  float gauss();
  float phillips(const vec2& k);

  int N, M;
  unsigned int displacemap;
  unsigned int gradientmap;

  vec2 W, scale;
  float A, L, V, G;

  float *normal;
  fftwf_plan_s * plan;
  fftwf_complex *h0, *ht, *hd;
};

#endif
