#include "Shader.h"

#include <Inventor/nodes/SoVertexShader.h>
#include <Inventor/nodes/SoFragmentShader.h>
#include <Inventor/nodes/SoShaderParameter.h>

Shader::Shader()
{
  // load shaders

  vert = new SoVertexShader;
  vert->sourceProgram = "VertexShader.glsl";
  vert->ref();

  frag = new SoFragmentShader;
  frag->sourceProgram = "FragmentShader.glsl";
  frag->ref();

  this->shaderObject.set1Value(0, vert);
  this->shaderObject.set1Value(1, frag);

  // initialize shader parameters

  SoShaderParameter1i * texunit0 = new SoShaderParameter1i;
  texunit0->name = "displacements";
  texunit0->value = 0;

  SoShaderParameter1i * texunit1 = new SoShaderParameter1i;
  texunit1->name = "displacements2";
  texunit1->value = 1;

  SoShaderParameter1i * texunit2 = new SoShaderParameter1i;
  texunit2->name = "normals";
  texunit2->value = 2;

  SoShaderParameter1i * texunit3 = new SoShaderParameter1i;
  texunit3->name = "sky";
  texunit3->value = 3;

  frag->parameter.set1Value(0, texunit0);
  frag->parameter.set1Value(1, texunit1);
  frag->parameter.set1Value(2, texunit2);
  frag->parameter.set1Value(3, texunit3);
}

/*
  Update shader with spatial scales for the two height fields
*/
void
Shader::setScales(SbVec2f scales)
{
  SoShaderParameter2f * s = new SoShaderParameter2f;
  s->name = "scales";
  s->value = scales;
  frag->parameter.set1Value(4, s);
}

Shader::~Shader()
{
  vert->unref();
  frag->unref();
}
