#ifndef WAVEFIELD_H
#define WAVEFIELD_H

#include <Inventor/fields/SoSFFloat.h>
#include <Inventor/fields/SoSFShort.h>
#include <Inventor/fields/SoSFUInt32.h>
#include <Inventor/fields/SoSFVec2f.h>
#include <Inventor/fields/SoSFVec2s.h>
#include <Inventor/nodes/SoSubNode.h>

class SoGLDisplayList;
class SoGLImage;
class SoSensor;
class SoFieldSensor;
class SoNodeSensor;
class TileMap;

class WaveField : public SoNode {
  SO_NODE_HEADER(WaveField);

public:
  SoSFVec2s size;      // determines the map resolution
  SoSFFloat amplifier; // wave field amplifier
  SoSFFloat gravity;   // gravitational constant
  SoSFVec2f wind;      // wind direction and magnitude
  SoSFFloat interval;  // time interval between updates
  SoSFVec2f scales;    // spatial scales of the two spectra
  SoSFUInt32 seed;     // random number generator seed

  WaveField();
  static void initClass();

  virtual void GLRender(SoGLRenderAction * action);

private:
  float t;
  bool resample;
  float * normal;
  TileMap * tile[2];

  SoGLDisplayList * dlist;
  SoGLImage * image;
  SoFieldSensor * sizesensor;
  SoNodeSensor * nodesensor;
  SoTimerSensor * timersensor;

  ~WaveField();

  void allocate();
  void deallocate();

  void sample();
  void shift(float t);
  void transform();

  static void resizeCB(void * closure, SoSensor * sensor);
  static void sampleCB(void * closure, SoSensor * sensor);
  static void shiftCB(void * closure, SoSensor * sensor);
};

#endif
